#include <gtest/gtest.h>
#include <cstdlib>
#include "solution.hpp"

TEST(DynamicMemory, createIntegerArray____singleElementRange) {
    try {
        srand(time(nullptr));
        int a = rand() % 100;
        int* p = my::createIntegerArray(a,a);

        // the call must return a non-zero address
        ASSERT_NE(p,nullptr);

        // an array must be an array contining a single value a
        EXPECT_EQ(p[0],a);

        // free allocated memory when we no longer need it
        delete[] p;
    } catch ( ... ) {
        // this call is not expected to throw
        FAIL();
    }
}

TEST(DynamicMemory, createIntegerArray____multipleElementRange) {
    try {
        srand(time(nullptr));
        int a = rand() % 40;
        int b = rand() % 60 + 41;  // a < b always
        int* p = my::createIntegerArray(a,b);

        // the call must return a non-zero address
        ASSERT_NE(p,nullptr);

        // verify the contents of the array
        for (int i=0; i<=b-a; i++) {
            EXPECT_EQ(p[i], a + i);
        }

        // free allocated memory
        delete[] p;
    } catch ( ... ) {
        // this call is not expected to throw
        FAIL();
    }
}

TEST(DynamicMemory, createIntegerArray____invalidRange) {
    try {
        srand(time(nullptr));
        int a = rand() % 40;
        int b = rand() % 60 + 41;  // a < b always
        int *p = my::createIntegerArray(b,a);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}


TEST(DynamicMemory, createCharacterArray1____singleElementRange) {
    try {
        srand(time(nullptr));
        char a = static_cast<char>(rand() % 256-128);
        char* p = my::createCharacterArray1(a,a);

        // the call must return a non-zero address
        ASSERT_NE(p,nullptr);

        // an array must be an array contining a single value a
        EXPECT_EQ(p[0],a);

        // free allocated memory when we no longer need it
        delete[] p;
    } catch ( ... ) {
        // this call is not expected to throw
        FAIL();
    }
}

TEST(DynamicMemory, createCharacterArray1____multipleElementRange) {
    try {
        srand(time(nullptr));
        char a = static_cast<char>(rand() % 40);
        char b = static_cast<char>(rand() % 60 + 41);  // a < b always
        char* p = my::createCharacterArray1(a,b);

        // the call must return a non-zero address
        ASSERT_NE(p,nullptr);

        // verify the contents of the array
        for (int i=0; i<=b-a; i++) {
        EXPECT_EQ(p[i], a + i);
        }

        // free allocated memory
        delete[] p;
    } catch ( ... ) {
        // this call is not expected to throw
        FAIL();
    }
}

TEST(DynamicMemory, createCharacterArray1____invalidRange) {
    try {
        srand(time(nullptr));
        char a = static_cast<char>(rand() % 40);
        char b = static_cast<char>(rand() % 60 + 41);  // a < b always
        char *p = my::createCharacterArray1(b,a);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, createCharacterArray2____singleElementRange) {
    try {
        srand(time(nullptr));
        short a = static_cast<short>(rand() % 256-128);
        char* p = my::createCharacterArray2(a,a);

        // the call must return a non-zero address
        ASSERT_NE(p,nullptr);

        // an array must be an array contining a single value a
        EXPECT_EQ(p[0],a);

        // free allocated memory when we no longer need it
        delete[] p;
    } catch ( ... ) {
        // this call is not expected to throw
        FAIL();
    }
}

TEST(DynamicMemory, createCharacterArray2____multipleElementRange) {
    try {
        srand(time(nullptr));
        short a = static_cast<short>(rand() % 40);
        short b = static_cast<short>(rand() % 60 + 41);  // a < b always
        char* p = my::createCharacterArray2(a,b);

        // the call must return a non-zero address
        ASSERT_NE(p,nullptr);

        // verify the contents of the array
        for (int i=0; i<=b-a; i++) {
        EXPECT_EQ(p[i], a + i);
        }

        // free allocated memory
        delete[] p;
    } catch ( ... ) {
        // this call is not expected to throw
        FAIL();
    }
}

TEST(DynamicMemory, createCharacterArray2____invalidRange) {
    try {
        srand(time(nullptr));
        short a = rand() % 40;
        short b = rand() % 60 + 41;  // a < b always
        char *p = my::createCharacterArray2(b,a);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, createMatrix____invalidInput1) {
    try {
        int data[5]{1,2,3,4,5};
        int** matrix = my::createMatrix(-1,-1,data);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, createMatrix____invalidInput2) {
    try {
        int data[5]{1,2,3,4,5};
        int** matrix = my::createMatrix(0,0,data);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, createMatrix____invalidInput3) {
    try {
        int** matrix = my::createMatrix(3,3,nullptr);
        // fail if the call doesn't throw
        FAIL();
    } catch ( const std::runtime_error& e ) {
        // expecting runtime_error
        SUCCEED();
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}

TEST(DynamicMemory, createMatrix____validInput) {
    try {
        int data[6]{1,2,3,4,5,6};
        int** matrix = my::createMatrix(2,3,data);

        // the call must return a non-zero address
        ASSERT_NE(matrix,nullptr);
        ASSERT_NE(matrix[0],nullptr);
        ASSERT_NE(matrix[1],nullptr);

        EXPECT_EQ(matrix[0][0],1);
        EXPECT_EQ(matrix[0][1],2);
        EXPECT_EQ(matrix[0][2],3);
        EXPECT_EQ(matrix[1][0],4);
        EXPECT_EQ(matrix[1][1],5);
        EXPECT_EQ(matrix[1][2],6);

        // free memory since we no longer need it
        for (int i=0; i<2; i++) {
            delete[] matrix[i];
        }
        delete[] matrix;
    } catch (...) {
        // the call is not expected to throw anything other than runtime_error
        FAIL();
    }
}
